function [aHOF, aHOFun] = computeAHOF(HOF, len, step, doNorm, nMags)
% [aHOF, aHOFun] = computeAHOF(HOF, len, step, doNorm, nMags)
% Computes aHOF descriptor given a HOF sequence.
%
% Input:
%  - HOF: matrix [nframes, ndims].
%  - len: number of frames to accumulate. E.g. 20
%  - step: every 'step' frames a aHOF descriptor is computed.
%  - doNorm: boolean to indicate whether to normalize aHOF or not. Def. false
%  - nMags: is doNorm = true, number of quantization intervals used for OF
%  magnitude during HOF computation. E.g. 4
%
% Output:
%  - aHOF: normalized aHOF descriptor in matrix [ndesc, ndims].
%  - aHOFun: unnormalized aHOF descriptor in matrix [ndesc, ndims].
%
% See also computeHOF, computePaHOFonUV
%
% (c) MJMJ/2012
%

if ~exist('doNorm', 'var')
   doNorm = false;
end

aHOFun = [];

slen = size(HOF,1);
final_ix = max(1, slen-len);
for ix = 1:step:final_ix,
   s = sum(HOF(ix:min(slen,ix+len-1),:),1); %sum(M(ix:ix+len-1,:),1);
   %s = s ./ sum(s);
   aHOFun = [aHOFun; s];      
end % ix


if doNorm
   aHOF = normalizeHistByMagnitude(aHOFun +1, nMags);
else
   aHOF = aHOFun;
end
