function HOF = computeHOF(U, V, gridconf, nAng, mags)
% HOF = computeHOF(U, V, gridconf,nAng, mags)
% Histograms of Optical Flow
%
% Input:
%  - U: u-component of Optical Flow [nrows, ncols, nframes]
%  - V: v-component of Optical Flow [nrows, ncols, nframes]
%  - gridconf: vector [ncells_X, ncells_Y]. E.g. [4,8]
%  - nAng: number of discretized angles. E.g. 8
%  - mags: vector with intervals for OF magnitude quantization. E.g. [0.5,1.5,2.5]
%
% See also histoFlujoRectBinFast
%
% (c) MJMJ/2011

nframes = size(U,3);

HOF = zeros(nframes, prod(gridconf)*nAng*(length(mags)+1), 'single');
gc1 = gridconf(1);
gc2 = gridconf(2);

for k = 1:nframes,
   
   [ H, foo ] = histoFlujoRectBinFast( U(:,:,k), V(:,:,k), nAng, gc1, gc2, mags );
   
   HOF(k,:) = single(H');
end % k
