function [ histograma,h2 ] = histoFlujoRectBinFast( u,v, nAng, XTiles, YTiles, thr )
% function [ histograma, h2 ] = histoFlujoRectBinFast( u,v, nAng, XTiles, YTiles, thr )
% Computes histogram of Optical Flow given components (u,v)
%
% TO DO: HELP MUST BE WRITTEN
% En esta version, cada vector vota siempre 1 en una unica celda y
% direccion.
%
%     Dado un par de matrices tridimensionales que representan el flujo,
%  se genera un histograma por direcciones de tam nAng, subdividiendo u y v
%  en XTilesxYTiles.
%
%  Input: 
%   - u,v      : 3D matrices (nrows, ncols, nframes) with optical flow components.
%   - nAng     : number of angles (bins) for quantization
%   - XTiles,YTiles : Subdivisiones a u,v. 
%
%   - thr: si es positivo, solo se tienen en cuenta los vectores cuyo modulo
%    supere el valor 'thr'. Si es negativo, se crean 2 bins de magnitud cuyo
%    valor limite es abs(thr)
%        Si es un vector, se crean bins de magnitud usando thr como
%        intervalos. Ej) [a, b, c] --> (-Inf,a](a,b](b,c](c,Inf)
%
%  Output:
%    - histograma : el propio histograma (nAng*XTiles*YTiles).
%    - h2: es matriz 3D o 4D, que no coincide con 'histograma' al hacer
%        un reshape directo. {YTiles, XTiles, nAng, nMag}
%
% Based on code of M. Lucena
% Modificado por mjmarin (27/05/2008)
% mjmarin (1/09/2008)
% mjmarin (24/11/2011): this version is based on histoFlujoRectBin.m 

if nargin < 6,
   thr = -Inf;
end  

nMag = 1;
if length(thr) > 1,
   nMag = 1+length(thr);   
   thr(end+1) = Inf; % Trick 
else
   if isfinite(thr) && thr < 0,
      nMag = 2;
      thr = abs(thr);
   end
end

[angulo,modulo] = cart2pol(u,v);
poshisto = 1 + floor((angulo+pi)*(nAng-1e-20)/(2*pi));

if XTiles==1 && YTiles==1

    histograma=zeros(nAng,1);
    for i=1:nAng
        histograma(i,1)=sum(((poshisto(:)==i).*modulo(:)));
    end
else

    histograma=zeros(nAng*XTiles*YTiles*nMag, 1, 'int16');

    ty=size(u,1);
    tx=size(u,2);
       
    % Define masks
    maskX=zeros(tx,XTiles);
    stepx = ceil(tx/XTiles);
    for x=1:XTiles,
        idx = 1+(x-1)*stepx:min(x*stepx, tx);
        maskX(idx,x)= 1;
    end
    
    maskY=zeros(ty,YTiles);
    stepy = ceil(ty/YTiles);
    for y=1:YTiles,
        idx = 1+(y-1)*stepy:min(y*stepy, ty);
        maskY(idx,y)= 1;
    end
        
    % Solo suman los que superen un umbral
    modmask = zeros(size(modulo,1), size(modulo,2), nMag);
    if nMag > 2,
       modmask(:,:,1) = modulo <= thr(1);
       for k = 1:nMag-1,
          modmask(:,:,k+1) = and(modulo > thr(k), modulo <= thr(k+1)); % mjmarin
       end
    else
       modmask = modulo >= thr; % mjmarin
    end
    
    if nMag > 1,
       if nMag == 2,
          sumValAng=zeros(ty,tx,nAng, nMag, 'int16');
          for i = 1:nAng,
             for k = 1:nMag,
                %sumValAng(:,:,i)=sum((poshisto==i).*modulo,3);
                if k == 1,
                   sumValAng(:,:,i, k)=sum((poshisto==i) .* modmask,3);
                else
                   sumValAng(:,:,i, k)=sum((poshisto==i) .* ~modmask,3);
                end
             end % k
          end
       else
          sumValAng=zeros(ty,tx,nAng, nMag, 'int16');
          for i = 1:nAng,
             poshi = poshisto==i;
             for k = 1:nMag,
                %sumValAng(:,:,i)=sum((poshisto==i).*modulo,3);
                sumValAng(:,:,i, k) = sum(poshi .* modmask(:,:,k),3);
             end % k
          end          
       end % > 2
    else
       sumValAng=zeros(ty,tx,nAng, 'int16');
       for i=1:nAng
          %sumValAng(:,:,i)=sum((poshisto==i).*modulo,3);
          sumValAng(:,:,i)=sum((poshisto==i) .* modmask,3);
       end
    end
    
    s = 0;
    if nMag > 1,
       h2 = zeros(YTiles, XTiles, nAng, nMag); % mjmarin       
    else
       h2 = zeros(YTiles, XTiles, nAng); % mjmarin
    end
    
    for x=1:XTiles
        vx = maskX(:,x)';
        for y=1:YTiles
            pesos = (maskY(:,y) * vx) > 0;
            for i=1:nAng,
               for k = 1:nMag,
                 %histograma(s+i,1)=sum(sum(sumValAng(:,:,i).*pesos));
                 %h2(y,x,i) = histograma(s+i,1); % mjmarin
                 
                 s = s+1;
                 
                 sva = sumValAng(:,:,i, k);
                 %v = sum(sva(pesos));
                 
                 histograma(s,1) = sum(sva(pesos)); %sum(sum(sumValAng(:,:,i, k).*pesos));
                 
                 h2(y,x,i, k) = histograma(s,1); % mjmarin                 
                    
               end % k
            end
            %s=s+nAng; % Ahora se incrementa en cada iteracion del bucle
            % mas interno
        end
    end

end
